/*
  This file is part of the KDE libraries
  Copyright (c) 2005 Willem Verschuur <willverschuur@yahoo.com>
  Copyright (c) 2006 Helio Chissini de Castro <helio@kde.org>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; see the file COPYING.LIB.  If not, write to
  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  Boston, MA 02110-1301, USA.
*/

/*
 * Lightweight openldap api wrappers, just makes the
 * whole memory management schlep so much easier
 *
 * test: exception handling.. try to break it
 *
 * note: dont pass these classes as results for functions..
 * for that you need to pull the variables into a counter
 * referenced core structure.
 *
 * todo: ldapnode class for a truer in-memory representation
 * of LDAP tree.. and easier query coding
 */

#ifndef _XLDAPLITE_H
#define _XLDAPLITE_H



#include <ldap.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qptrlist.h>



class xLDAPquery;
class xLDAPiterator;
class xLDAPattribute;



class xLDAPconnection
{

    
    friend class xLDAPquery;
    friend class xLDAPiterator;
    friend class xLDAPattribute;

    
    public:


        // read /etc/ldap.conf 
        xLDAPconnection();


        // free resources on object destruction
        ~xLDAPconnection();


        // connect to ldap server
        bool connect();


        // free resources
        void disconnect();


        // accessor
        const LDAP* connection()  { return m_Connection; }
        const QString& base() { return m_Base; }
        const QString& password() { return m_RootPassword; }


    protected:


        LDAP*           m_Connection;
        QString         m_RootPassword;
        ber_int_t       m_Scope;
        QString         m_Base;
       

};



class xLDAPquery
{


    // iterator should have access to members
    friend class xLDAPiterator;
    friend class xLDAPattribute;

    
    public:


        // execute a query
        xLDAPquery( xLDAPconnection&, 
                    const QString& filter, 
                    const QString& base = 0, 
                    char** attrs = 0 );


        // free resources on object destruction
        ~xLDAPquery();

        
    protected:


        xLDAPconnection*    m_Connection;
        LDAPMessage*        m_Query;
        int                 m_Result;

        
};



class xLDAPiterator
{


    friend class xLDAPattribute;

    
    public:


        xLDAPiterator(xLDAPquery&);


        ~xLDAPiterator();


        operator bool(void);


        void operator ++ (void);


    protected:


        void clearElement();

        
        xLDAPquery*     m_Query;
        LDAPMessage*    m_Entry;
        BerElement*     m_Element; // used by attribute functions


};



class xLDAPattribute
{

    
    public:


        xLDAPattribute(xLDAPiterator&);


        ~xLDAPattribute();


        operator bool(void);


        void operator ++ (void);


        const char* attribute() { return m_Attribute; }


        const char* firstValue();
        const char* nextValue();

        
    protected:


        void clearAttribute();
        void clearValues();


        xLDAPiterator*          m_Iterator;
        char*                   m_Attribute;    // attribute
        char**                  m_Values;
        char**                  m_CurrentValue;

       
};



#endif

