/* This file is part of the KDE libraries
   Copyright (C) 2000 David Faure <faure@kde.org>
   Copyright (C) 2007 Per Øyvind Karlsen <peroyvind@mandriva.org>


   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include <config.h>

#if defined( HAVE_LZMA_SUPPORT )
// we don't need that
#define LZMADEC_NO_STDIO
#include <sys/types.h>
extern "C" {
	#include <lzmadec.h>
}

#include <kdebug.h>
#include <klibloader.h>

#include "klzmafilter.h"

class KLzmaFilterFactory : public KLibFactory
{
public:
    KLzmaFilterFactory() : KLibFactory() {}
    virtual ~KLzmaFilterFactory(){}
    QObject *createObject( QObject *, const char *, const char*, const QStringList & )
    {
        return new KLzmaFilter;
    }
};

K_EXPORT_COMPONENT_FACTORY( klzmafilter, KLzmaFilterFactory )

// Not really useful anymore
class KLzmaFilter::KLzmaFilterPrivate
{
public:
    lzmadec_stream zStream;
};

KLzmaFilter::KLzmaFilter()
{
    d = new KLzmaFilterPrivate;
    d->zStream.lzma_alloc = 0;
    d->zStream.lzma_free = 0;
    d->zStream.opaque = 0;
    m_mode = 0;
}


KLzmaFilter::~KLzmaFilter()
{
    delete d;
}

void KLzmaFilter::init( int mode )
{
    d->zStream.next_in = 0;
    d->zStream.avail_in = 0;
    if ( mode == IO_ReadOnly )
    {
        // Ignoring error like kgzipfilter and kbzip2filter.
        // Like with zlib and libbzip2, we will safely fail if we
	// have liblzmadec 27dc3a939f64f97a4f02bd9d06b1e901e90f3571
	// or later (will be in LZMA Utils 4.32.4).
        (void)lzmadec_init(&d->zStream);
    } else
        kdWarning(7129) << "Unsupported mode " << mode << ". Only IO_ReadOnly supported" << endl;
    m_mode = mode;
    m_finishing = false;
}

void KLzmaFilter::terminate()
{
    if ( m_mode == IO_ReadOnly )
    {
        int result = lzmadec_end(&d->zStream);
        kdDebug(7129) << "lzmadec_end returned " << result << endl;
    } else
        kdWarning(7129) << "Unsupported mode " << m_mode << ". Only IO_ReadOnly and IO_WriteOnly supported" << endl;
}


void KLzmaFilter::reset()
{
    kdDebug(7129) << "KLzmaFilter::reset" << endl;
    // lzma doesn't seem to have a reset call...
    terminate();
    init( m_mode );
}

void KLzmaFilter::setOutBuffer( char * data, uint maxlen )
{
    d->zStream.avail_out = maxlen;
    d->zStream.next_out = (uint8_t *) data;
}

void KLzmaFilter::setInBuffer( const char *data, unsigned int size )
{
    if (size == 0)
        m_finishing = true;

    d->zStream.avail_in = size;
    d->zStream.next_in = (uint8_t *)const_cast<char *>(data);
}

int KLzmaFilter::inBufferAvailable() const
{
    return d->zStream.avail_in;
}

int KLzmaFilter::outBufferAvailable() const
{
    return d->zStream.avail_out;
}

KLzmaFilter::Result KLzmaFilter::uncompress()
{
    int result = lzmadec_decode(&d->zStream, m_finishing);
    if ( result != LZMADEC_OK )
    {
        kdDebug(7129) << "lzmadec_decode returned " << result << endl;
        kdDebug(7129) << "KLzmaFilter::uncompress " << ( result == LZMADEC_OK ? OK : ( result == LZMADEC_STREAM_END ? END : ERROR ) ) << endl;
    }

    switch (result) {
        case LZMADEC_OK:
                return OK;
        case LZMADEC_STREAM_END:
                return END;
        default:
                return ERROR;
    }
}

KLzmaFilter::Result KLzmaFilter::compress( bool finish )
{
	kdError(7129) << "lzmadec doesn't support compression!" << finish << endl;	
	return ERROR;
}	
#endif
